require 'time'
require 'base64'
require 'openssl'
require 'securerandom'
require 'rest-client'
require 'simple-hmac'
require 'json'

module DistributionApi
  class << self
    attr_accessor :api_endpoint, :api_key, :api_secret
  end

  def self.setup(api_key, api_secret, api_endpoint='https://distribution.api.wizypay.com/v1')
    self.api_endpoint   = api_endpoint
    self.api_key    = api_key
    self.api_secret = api_secret
  end

  def self.create_card(reference, amount, currency, merchant_id, user)
    user       = user.to_json if user.is_a?(Hash)
    payload    = { amount: amount, currency: currency, merchant_id: merchant_id, reference: reference, user: user }
    req_params = { url: "#{api_endpoint}/cards", method: :post, headers: { accept: :json }, payload: payload }
    req        = RestClient::Request.new(req_params)
    req.sign!(api_key, api_secret)
    parse_response req.execute
  end

  def self.cancel_card(reference)
    req_params = { url: ("#{api_endpoint}/cards/#{URI::encode_www_form_component reference}/cancel"), method: :post, headers: { accept: :json }, payload: {} }
    req        = RestClient::Request.new(req_params)
    req.sign!(api_key, api_secret)
    req.execute
  end

  def self.card_url(reference)
    timestamp    = Time.now.utc.httpdate
    security     = Base64.strict_encode64(OpenSSL::HMAC.digest('sha256', api_secret, "#{reference}\n#{timestamp}"))
    query_string = {
        key:       api_key,
        timestamp: timestamp,
        reference: reference,
        security:  security
    }.map { |k, v| "#{k}=#{URI::encode_www_form_component(v)}" }.join('&')
    "#{api_endpoint}/widget?#{query_string}"
  end

  private

  def self.parse_response(response)
    Hash[JSON::parse(response).map { |k,v| [k.to_sym, v] }]
  end
end
